/*
 * JBoss, Home of Professional Open Source
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */ 
package org.jboss.ha.client.loadbalance;

import java.util.List;

import org.jboss.ha.framework.interfaces.FamilyClusterInfo;



/**
 * LoadBalancePolicy implementation that always favors the next available
 * target in the list, ensuring maximum load balancing always occurs.
 * <p>
 * Different instances of this class will coordinate their target
 * selection via the 
 * {@link FamilyClusterInfo#getCursor() FamilyClusterInfo cursor}, so if
 * proxy A makes the first call to target 1, and proxy B makes the next 
 * call, it will invoke on target 2; the third call will go to
 * target 3 no matter which proxy makes it, and so on. 
 * </p>
 * <p>
 * The first target in the round-robin is chosen randomly, ensuring that
 * proper load balancing occurs when many clients in different VM's download
 * the proxy.
 * </p>
 *
 * @see org.jboss.ha.client.loadbalance.LoadBalancePolicy
 * @see org.jboss.ha.framework.interfaces.FamilyClusterInfo#getCursor()
 *
 * @author <a href="mailto:sacha.labourey@cogito-info.ch">Sacha Labourey</a>.
 * @author <a href="mailto:bill@burkecentral.com">Bill Burke</a>.
 * @version $Revision: 57642 $
 */
public class RoundRobin implements LoadBalancePolicy
{
   // Constants -----------------------------------------------------
   /** @since 1.3.4.2 */
   private static final long serialVersionUID = 8660076707279597114L;

   // Attributes ----------------------------------------------------
   
   // Static --------------------------------------------------------
   
   // Constructors --------------------------------------------------
       
   // Public --------------------------------------------------------
   
   public Object chooseTarget (FamilyClusterInfo clusterFamily)
   {
      int cursor = clusterFamily.getCursor ();
      List targets = clusterFamily.getTargets ();

      if (targets.size () == 0)
         return null;
      
      if (cursor == FamilyClusterInfo.UNINITIALIZED_CURSOR)
      {         
         // Obtain a random index into targets
         cursor = RandomRobin.localRandomizer.nextInt(targets.size());
      }
      else
      {
         // Choose the next target
         cursor = ( (cursor + 1) % targets.size() );
      }
      clusterFamily.setCursor (cursor);

      return targets.get(cursor);
   }

}
