/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.test.ha.client.loadbalance;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import junit.framework.TestCase;

import org.jboss.ha.client.loadbalance.FirstAvailable;
import org.jboss.ha.client.loadbalance.LoadBalancePolicy;
import org.jboss.ha.framework.interfaces.ClusteringTargetsRepository;
import org.jboss.ha.framework.interfaces.FamilyClusterInfo;

/**
 * Tests of the FirstAvailable load balance policy.
 * 
 * @author Brian Stansberry
 */
public class FirstAvailableUnitTestCase extends TestCase
{
   protected static final String[] TARGETS = { "A", "B", "C" };
   protected static final String FAMILY_BASE = "testFamily";
   
   protected static int testCount = 0;
   
   /**
    * Create a new FirstAvailableUnitTestCase.
    * 
    * @param name
    */
   public FirstAvailableUnitTestCase(String name)
   {
      super(name);
   }
   
   protected LoadBalancePolicy getLoadBalancePolicy()
   {
      return new FirstAvailable();
   }
   
   public void testBasicStickiness()
   {
      testCount++;
      List<String> targets = Arrays.asList(TARGETS);
      FamilyClusterInfo fci = ClusteringTargetsRepository.initTarget(FAMILY_BASE + testCount, targets);
      
      LoadBalancePolicy lbp = getLoadBalancePolicy();
      Object target = lbp.chooseTarget(fci);
      
      assertTrue(targets.contains(target));
      
      for (int i = 0; i < 10; i++)
         assertEquals(target, lbp.chooseTarget(fci));
   }
   
   public void testInitialSelection()
   {
      testCount++;
      FamilyClusterInfo fci = ClusteringTargetsRepository.initTarget(FAMILY_BASE + testCount, Arrays.asList(TARGETS));
      
      Set<Object> selected = new HashSet<Object>();
      
      for (int i = 0; i < 10000; i++)
      {
         LoadBalancePolicy lbp = getLoadBalancePolicy();
         selected.add(lbp.chooseTarget(fci));
         if (selected.size() == TARGETS.length)
            break;
      }
      
      assertEquals(TARGETS.length, selected.size());
   }
   
   public void testStickinessAfterFailover()
   {
      testCount++;
      List<String> targets = Arrays.asList(TARGETS);
      FamilyClusterInfo fci = ClusteringTargetsRepository.initTarget(FAMILY_BASE + testCount, targets);
      
      LoadBalancePolicy lbp = getLoadBalancePolicy();
      Object firstTarget;
      Object target = firstTarget = lbp.chooseTarget(fci);
      
      assertTrue(targets.contains(target));
      
      for (int i = 0; i < 5; i++)
         assertEquals(target, lbp.chooseTarget(fci));
      
      fci.removeDeadTarget(target);
      Object secondTarget;
      target = secondTarget = lbp.chooseTarget(fci);
      
      assertFalse(firstTarget.equals(secondTarget));
      assertTrue(targets.contains(target));
      
      for (int i = 0; i < 5; i++)
         assertEquals(target, lbp.chooseTarget(fci));
      
      fci.removeDeadTarget(target);
      Object thirdTarget;
      target = thirdTarget = lbp.chooseTarget(fci);
      
      assertFalse(firstTarget.equals(thirdTarget));
      assertFalse(secondTarget.equals(thirdTarget));
      assertTrue(targets.contains(target));
      
      for (int i = 0; i < 5; i++)
         assertEquals(target, lbp.chooseTarget(fci));
      
      fci.removeDeadTarget(target);
      
      assertNull(lbp.chooseTarget(fci));
   }
   
   public void testNoAvailableTargets()
   {
      testCount++;
      FamilyClusterInfo fci = ClusteringTargetsRepository.initTarget(FAMILY_BASE + testCount, new ArrayList<Object>());
      
      assertNull(getLoadBalancePolicy().chooseTarget(fci));
   }
   
   

}
